Module: RuboCopMethodOrder
- Defined in:
- lib/rubocop_method_order/version.rb,
lib/rubocop_method_order/method_collector.rb,
lib/rubocop_method_order/method_node_collection.rb,
lib/rubocop_method_order/public_method_node_collection.rb
Overview
Version information for the gem.
Defined Under Namespace
Modules: VERSION Classes: MethodCollector, MethodNodeCollection, PublicMethodNodeCollection
Class Method Summary collapse
-
.gem_version ⇒ Object
Returns the version of the currently loaded gem as a Gem::Version.
Class Method Details
.gem_version ⇒ Object
Returns the version of the currently loaded gem as a Gem::Version
6 7 8 |
# File 'lib/rubocop_method_order/version.rb', line 6 def self.gem_version Gem::Version.new VERSION::STRING end |